#include "capd/capdlib.h"
#include <iostream>
using namespace std;
using namespace capd;
using namespace capd::matrixAlgorithms;

int NewtonMethod()
{
	interval a=interval(14)/interval(10);
	interval b=interval(3)/interval(10);
	
	IMap F= "par:a,b;"
			"var:x,y;"
			"fun:1-a*x^2+y-x,b*x-y;";
	
	F.setParameter("a",a);
	F.setParameter("b",b);
	
	IVector x0(2);
	x0[0] = sqrt(609)/28-0.25;
	x0[1] = b*x0[0];
	cout << "x0=" << x0 << "    F(x0)=" << F(x0) << endl;
	
	IVector B(2);
	B[0] = x0[0] + interval(-1,1)*power(10,-14);
	B[1] = x0[1] + interval(-1,1)*power(10,-14);
	
	IVector N = x0 - gauss( F[B], F(x0));
	if( subsetInterior(N,B) ) return 1;
	return 0;
}

void obtainGuess(IVector &x0,IMatrix &C)
{
	double a=1.4;
	double b=0.3;
	
	DMap F= "par:a,b;"
			"var:x,y;"
			"fun:1-a*x^2+y-x,b*x-y;";
	
	F.setParameter("a",a);
	F.setParameter("b",b);
	
	// initial guess:
	DVector x(2);
	x[0] = 0.5;
	x[1] = 0.5;
	
	// Newton operator (non-rigorous)
	for(int i=0;i<5;i++)
	{
		cout << "x=" << x << "   F(x)=" <<F(x) << endl;
		x = x - gauss(F[x],F(x));
	}

	x0=IVector(x);
	C =IMatrix(gaussInverseMatrix(F[x]));
}

int KrawczykMethod()
{
	interval a=interval(14)/interval(10);
	interval b=interval(3)/interval(10);
	
	IMap F= "par:a,b;"
			"var:x,y;"
			"fun:1-a*x^2+y-x,b*x-y;";
	
	F.setParameter("a",a);
	F.setParameter("b",b);
	
	IVector x0;
	IMatrix C;
	obtainGuess(x0,C);
	
	IMatrix Id(2,2);
	Id[0][0]=interval(1);
	Id[1][1]=interval(1);
	
	IVector B(2);
	B[0] = x0[0] + interval(-1,1)*power(10,-14);
	B[1] = x0[1] + interval(-1,1)*power(10,-14);
	
	IVector K = x0 - C*F(x0) + (Id - C*F[B])*(B-x0);
	if( subsetInterior(K,B) ) return 1;
	return 0;
}

int main()
{
  	try
    {	
		cout << endl; cout.precision(4);
		
        if(NewtonMethod()  ==1) cout << "Newton:   successful" << endl << endl;
		
		//IVector x0;
		//IMatrix C;
		//obtainGuess(x0,C);
		
		//if(KrawczykMethod()==1) cout << "Krawczyk: successful" << endl << endl;
    }
  	catch(exception& e)
    {
      cout << endl << "Exception caught: " << e.what() << endl;
    }
    return 0;
}