#include "capd/capdlib.h"
#include <iostream>
using namespace std;
using namespace capd;
using namespace capd::matrixAlgorithms;

void plot(IVector x,int i)
{
	ofstream file("set"+to_string(i)+".txt");
	file << x[0].leftBound()  << " " << x[1].leftBound()  << endl;
	file << x[0].rightBound() << " " << x[1].leftBound()  << endl;
	file << x[0].rightBound() << " " << x[1].rightBound() << endl;
	file << x[0].leftBound()  << " " << x[1].rightBound() << endl;
	file << x[0].leftBound()  << " " << x[1].leftBound()  << endl;	
	file.close();
}

void showWrappingEffect(double eps)
{
	IMap f = "par:s,c,eps;"
			 "var:x,y;"
			 "fun:c*x+s*y+eps*x^2,-s*x+c*y+eps*x*y;";
	double a=-0.5;
	f.setParameter("s",sin(a));
	f.setParameter("c",cos(a));
	f.setParameter("eps",eps);
	
	double b=0.1;
	IVector x(2);
	x[0] = interval(1.0-b,1.0+b);
	x[1] = x[0]-1.0;
	
	int n=10;
	for(int i=0;i<n;i++)
	{
		plot(x,i);
		x=f(x);
	}
}

void plot(IVector x,IMatrix A,IVector Y,int i)
{
	ofstream file("set"+to_string(i)+".txt");
	IVector B,y;
	
	B=Y; B[0]=B[0].left(); B[1]=B[1].left(); y=x+A*B;
	file << y[0].leftBound()  << " " << y[1].leftBound()  << endl;
	
	B=Y; B[0]=B[0].right(); B[1]=B[1].left(); y=x+A*B;
	file << y[0].leftBound()  << " " << y[1].leftBound()  << endl;
	
	B=Y; B[0]=B[0].right(); B[1]=B[1].right(); y=x+A*B;
	file << y[0].leftBound()  << " " << y[1].leftBound()  << endl;
	
	B=Y; B[0]=B[0].left(); B[1]=B[1].right(); y=x+A*B;
	file << y[0].leftBound()  << " " << y[1].leftBound()  << endl;
	
	B=Y; B[0]=B[0].left(); B[1]=B[1].left(); y=x+A*B;
	file << y[0].leftBound()  << " " << y[1].leftBound()  << endl;
		
	file.close();
}

void reduceWrappingEffect(double eps)
{
	// this is our map:
	IMap f = "par:s,c,eps;"
			 "var:x,y;"
			 "fun:c*x+s*y+eps*x^2,-s*x+c*y+eps*x*y;";
	double a=-0.5;
	f.setParameter("s",sin(a));
	f.setParameter("c",cos(a));
	f.setParameter("eps",eps);
	
	// this is the rotation matrix
	IMatrix R(2,2);
	R[0][0]=cos(a);  R[0][1]=sin(a);
	R[1][0]=-sin(a); R[1][1]=cos(a);
	
	int n=10;
	// these will be our local coordinates
	vector<IMatrix> A(n);
	// A[0] is the identity
	A[0]=IMatrix(2,2);
	A[0][0][0]=interval(1); A[0][1][1]=interval(1); 
	
	// boxes for our set representation
	vector<IVector> x(n), Y(n), X(n);
	
	// this is our initial set
	double b=0.1;
	X[0] = IVector(2);
	X[0][0] = interval(1.0-b,1.0+b);
	X[0][1] = X[0][0]-1.0;
	
	// this is the middle of X[0] 
	x[0]=midVector(X[0]);
	// We choose Y[0] so that X[0] = x[0] + Y[0]:
	Y[0]=X[0]-x[0];
	
	plot(X[0],0);
	for(int k=0;k<n-1;k++)
	{
		x[k+1]=f(x[k]);
		A[k+1]=R*A[k];
		Y[k+1]=( gaussInverseMatrix(A[k+1])*f[X[k]]*A[k] )*Y[k];
		X[k+1]=x[k+1] + A[k+1]*Y[k+1];
		plot(x[k+1],A[k+1],Y[k+1],k+1);
	}
}

int main()
{
  	try
    {	
		double eps=0.1;
		showWrappingEffect(eps);
		//reduceWrappingEffect(eps);
    }
  	catch(exception& e)
    {
      cout << endl << "Exception caught: " << e.what() << endl;
    }
    return 0;
}